SystemOrganization addCategory: #'Bindings-Base'!
SystemOrganization addCategory: #'Bindings-Tests'!

Morph subclass: #BindingsTestConsumer
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestConsumer methodsFor: 'accessing' stamp: 'topa 12/29/2009 11:51'!
value

	^ value! !

!BindingsTestConsumer methodsFor: 'accessing' stamp: 'topa 12/29/2009 11:52'!
value: anObject

	value := anObject! !

Morph subclass: #BindingsTestObject
	instanceVariableNames: 'a b'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:15'!
a
	
	<bindable>
	^ self valueOfVariableNamed: #a! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:15'!
a: anObject
	
	<bindable>
	self setVariableNamed: #a toValue: anObject! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:16'!
b

	^ b! !

!BindingsTestObject methodsFor: 'accessing' stamp: 'topa 12/27/2009 13:16'!
b: anObject

	b := anObject! !

Morph subclass: #BindingsTestProducer
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTestProducer methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:20'!
value

	^ value! !

!BindingsTestProducer methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:20'!
value: anObject

	value := anObject.! !

!Morph class methodsFor: '*bindings' stamp: 'topa 12/29/2009 13:41'!
bindingsManagerClass

	^ BindingsManager! !

!Morph class methodsFor: '*bindings' stamp: 'topa 12/27/2009 13:22'!
exposedBindings
	"exposed Bindings are Methods that have the #bindable pragma"
	
	^ Array streamContents: [ :stream |
		(Pragma allNamed: #bindable in: self) do: [:pragma |
			stream nextPut: pragma selector]]
! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/27/2009 17:56'!
bind: bindable to: aMorph with: aKeyPath as: key
	"Binds the bindable of the current Morph to
	a model described by model key path"
	key == #value ifTrue: [
		"Check if name of exposed variable is inside this morph"
		(self exposedBindings includes: bindable) ifTrue: [
			"Inject getter/setter for exposed variable (that updates other notifiers) if not existent"
			"Remember morph and keypath"
			(self bindings startObservingKey: bindable notifying: aMorph with: aKeyPath).
			"Init notification mechanism at other morph (object notifies morph notifies me)"
			"set KVO-compliant method (getter/setter to call) as delegate for notification"]].
	key == #collection ifTrue: [^'collection'].
! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/29/2009 13:41'!
bindings

	(self assureExtension hasProperty: #bindingsManager) 
		ifFalse: [self assureExtension 
			setProperty: #bindingsManager 
			toValue: (self class bindingsManagerClass new target: self)].
		
	^ self assureExtension valueOfProperty: #bindingsManager! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/27/2009 16:07'!
exposedBindings

	^ self class exposedBindings
! !

!Morph methodsFor: '*bindings' stamp: 'AK 12/24/2009 15:44'!
notifyMorphsForChangesAt: aKeyPath
	"takes registrations to send objects notifications if keypath has changed
	(changed object informs this morph with this method)"! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/29/2009 18:56'!
setVariableNamed: aSymbol toValue: anObject

	(anObject = (self instVarNamed: aSymbol))
		ifFalse: [
			self bindings triggerEvent: #aboutToChange with: (self -> aSymbol).
			self instVarNamed: aSymbol put: anObject.
			self bindings triggerEvent: #hasChanged with: (self -> aSymbol)].! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/27/2009 18:02'!
unbind: bindable from: aMorph with: aKeyPath

	(self exposedBindings includes: bindable) ifTrue: [
		"Remember morph and keypath"
		(self bindings stopObservingKey: bindable regarding: aMorph with: aKeyPath)]! !

!Morph methodsFor: '*bindings' stamp: 'topa 12/29/2009 13:52'!
valueOfVariableNamed: aSymbol

	self bindings triggerEvent: #accessed with: aSymbol.
	^ self instVarNamed: aSymbol! !

Object subclass: #BindingsDescriptor
	instanceVariableNames: 'origin target bindable keyPath'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsDescriptor class methodsFor: 'instance creation' stamp: 'topa 12/29/2009 16:50'!
origin: aMorph
bindable: aSymbol
target: anotherMorph
keyPath: aKeyPath

	^ self new
		origin: aMorph;
		bindable: aSymbol;
		target: anotherMorph;
		keyPath: aKeyPath! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 12/29/2009 16:55'!
= aBindingsDescriptor

	^ self == aBindingsDescriptor
		ifTrue: [true]
		ifFalse:[
			(self origin     = aBindingsDescriptor origin) and: [
			(self bindable = aBindingsDescriptor bindable) and: [
			(self target    = aBindingsDescriptor target) and: [
			(self keyPath  = aBindingsDescriptor keyPath)]]]]
			! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
bindable

	^ bindable! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
bindable: anObject

	bindable := anObject.! !

!BindingsDescriptor methodsFor: 'comparing' stamp: 'topa 12/29/2009 17:00'!
hash

	^ (((self origin hash 
		bitXor: self bindable hash) 
			bitXor: self target hash)
				bitXor: self keyPath hash)! !

!BindingsDescriptor methodsFor: 'testing' stamp: 'topa 12/29/2009 17:01'!
isBindingDescriptor

	^ true! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
keyPath

	^ keyPath! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
keyPath: anObject

	keyPath := anObject.! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:36'!
origin

	^ origin! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:34'!
origin: anObject

	origin := anObject.! !

!BindingsDescriptor methodsFor: 'as yet unclassified' stamp: 'topa 12/29/2009 18:43'!
printOn: aStream

	super printOn: aStream.
	aStream
		nextPut: $(;
		print: self origin; nextPutAll: ''; print: self bindable;
		nextPutAll: ' -> ';
		print: self target; nextPutAll: ''; print: self keyPath;
		nextPut: $)! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:36'!
target

	^ target! !

!BindingsDescriptor methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:35'!
target: anObject

	target := anObject.! !

Object subclass: #BindingsManager
	instanceVariableNames: 'observedBindings target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Base'!

!BindingsManager methodsFor: 'reacting' stamp: 'topa 12/29/2009 18:48'!
aboutToChange: aBindingDescriptor! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 19:08'!
bindingDescriptorForAssociation: aBindingAssoc

	|  target bindable |
	target := aBindingAssoc key.
	bindable := aBindingAssoc value.

	^ (self observedKey: bindable) 
		detect: [:bd| 
			(bd target = target) and: [
			(bd bindable = bindable) and: [
			bd origin = self target]]]! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/29/2009 17:19'!
forgetBinding: aBindingsDescriptor

	(self observedKey: aBindingsDescriptor bindable) 
		remove: aBindingsDescriptor ifAbsent: ["fallthough"].
	self removeIfObsoleteKey: aBindingsDescriptor bindable.
! !

!BindingsManager methodsFor: 'reacting' stamp: 'topa 12/29/2009 19:15'!
hasChanged: aBindingAssoc

	| descriptor |
	descriptor := self bindingDescriptorForAssociation: aBindingAssoc.

	" simple case"
	self target
		setVariableNamed: descriptor keyPath
		toValue: (descriptor target valueOfVariableNamed: descriptor bindable).! !

!BindingsManager methodsFor: 'initialize-release' stamp: 'topa 12/27/2009 18:12'!
initialize

	self observedBindings: Dictionary new.! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 12/29/2009 18:55'!
installEventHandlers: aBindingsDescriptor

	#( aboutToChange hasChanged ) do: [:eventSymbol |
		aBindingsDescriptor target bindings 
			when: eventSymbol
			send: eventSymbol asMutator
			to: aBindingsDescriptor origin bindings].! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/27/2009 14:17'!
observedBindings

	^ observedBindings! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/27/2009 14:17'!
observedBindings: anObject

	observedBindings := anObject.! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 16:58'!
observedKey: aKey

	^ self observedBindings at: aKey ifAbsentPut: [Set new]! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 12/29/2009 17:12'!
recordBinding: aBindingsDescriptor

	(self observedKey: aBindingsDescriptor bindable) 
		add: aBindingsDescriptor.! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/29/2009 17:19'!
removeEventHandlers: aBindingsDescriptor

	#( aboutToChange hasChanged ) do: [:eventSymbol |
			aBindingsDescriptor target bindings 
				removeActionsWithReceiver: (aBindingsDescriptor origin bindings) 
				forEvent: eventSymbol].
! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/27/2009 18:10'!
removeIfObsoleteKey: aKey

	((self observedBindings includesKey: aKey) and: [
		(self observedBindings at: aKey) isEmpty])
			ifTrue: [	self observedBindings removeKey: aKey]! !

!BindingsManager methodsFor: 'establishing' stamp: 'topa 12/29/2009 17:10'!
startObservingKey: bindable notifying: aMorph with: aKeyPath

	| descriptor |
	descriptor := (BindingsDescriptor origin: (self target) bindable: bindable target: aMorph keyPath: aKeyPath).
	self
		recordBinding: descriptor;
		installEventHandlers: descriptor.! !

!BindingsManager methodsFor: 'termination' stamp: 'topa 12/29/2009 17:21'!
stopObservingKey: bindable regarding: aMorph with: aKeyPath

	| descriptor |
	descriptor := (BindingsDescriptor origin: (self target) bindable: bindable target: aMorph keyPath: aKeyPath).

	(self observedBindings includesKey: bindable) ifTrue: [
		self 
			forgetBinding: descriptor;
			removeEventHandlers: descriptor]! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 12:45'!
target

	^ target! !

!BindingsManager methodsFor: 'accessing' stamp: 'topa 12/29/2009 12:45'!
target: aMorph

	target := aMorph! !

!Object methodsFor: '*bindings' stamp: 'topa 12/29/2009 17:01'!
isBindingDescriptor

	^ false! !

TestCase subclass: #BindingsTestCase
	instanceVariableNames: 'sut helper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

BindingsTestCase subclass: #BindingsDescriptorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsDescriptorTest methodsFor: 'running' stamp: 'topa 12/29/2009 18:19'!
setUp

	super setUp.
	self
		sut: BindingsDescriptor new;
		helper: BindingsTestObject new.! !

!BindingsDescriptorTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:22'!
testDescriptorCreation

	| bindings |
	self helper bind: #a to: self helper with: #a as: #value.
	
	bindings := self helper bindings observedBindings at: #a.
	bindings do: [:binding | 
		self should: [binding isBindingDescriptor]
				description: 'Binding descriptors should be created automatically upon binding creation'].
	
	self helper unbind: #a from: self helper with: #a.! !

!BindingsDescriptorTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:22'!
testDescriptorEquality

	| comparand |
	self sut 
		origin: self helper;
		bindable: #a;
		target: self helper;
		keyPath: #a.
		
	comparand := BindingsDescriptor origin: self helper bindable: #a target: self helper keyPath: #a.
	
	self should: [(self sut = comparand) and: [self sut hash = comparand hash]]
			description: 'Content-equal Bindings-Descriptors should be treated as equal';
		shouldnt: [self sut == comparand]
			description: 'Content-equal Bindings-Descriptors should not be treated as identical'.
	! !

BindingsTestCase subclass: #BindingsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Bindings-Tests'!

!BindingsTest methodsFor: 'running' stamp: 'topa 12/29/2009 18:19'!
setUp

	super setUp.
	self 
		sut: BindingsTestObject new;
		helper: BindingsTestObject new.! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:21'!
testCreateRemoveBindings
	
	| numOfBindings |
	numOfBindings := self sut bindings observedBindings size.
	
	self sut bind: #a to: self helper with: #a as: #value.	
	self should: [self sut bindings observedBindings size = (numOfBindings + 1)]
		description: 'Binding Morphs should increase the size of the observed bindings'.
	self should: [self sut bindings target = self sut]
		description: 'Binding Manager should know its target.'.
	

	self sut unbind: #a from: self helper with: #a.
	self should: [self sut bindings observedBindings size = numOfBindings]
		description: ' Unbinding Morphs should decrease the size of the observed bindings'.	
! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 12/29/2009 18:21'!
testExposedBindings

	self 
		should: [self sut class exposedBindings includes: #a];
		should: [self sut class exposedBindings includes: #a:];
		should: [self sut class exposedBindings size = 2];
		shouldnt: [self sut class exposedBindings includes: #b];
		shouldnt: [self sut class exposedBindings includes: #b:].
	! !

!BindingsTest methodsFor: 'testing' stamp: 'topa 12/29/2009 19:16'!
testReadBinding
	
	| theValue |
	theValue := 42.
	self sut bind: #a to: self helper with: #a as: #value.
	self helper a: theValue.	
	self should: [self sut a = theValue]
		description: 'Bound Morphs should have a value set by the binding target'.
	
	self sut unbind: #a from: self helper with: #a.
! !

!BindingsTestCase class methodsFor: 'testing' stamp: 'topa 12/29/2009 18:24'!
isAbstract

	^self name = #BindingsTestCase! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
helper

	^ helper! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
helper: aTestHelperObject

	helper := aTestHelperObject.! !

!BindingsTestCase methodsFor: 'running' stamp: 'topa 12/29/2009 18:16'!
setUp

	super setUp.
	self
		sut: nil;
		helper: nil.! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:15'!
sut

	^ sut! !

!BindingsTestCase methodsFor: 'accessing' stamp: 'topa 12/29/2009 18:14'!
sut: aSystemUnderTest

	sut := aSystemUnderTest.! !

!BindingsTestCase methodsFor: 'running' stamp: 'topa 12/29/2009 18:13'!
tearDown

	self
		sut: nil;
		helper: nil.
	super tearDown! !
